#include "Episode.h"

Episode::Episode(Segment *parentSegment)
  : Segment(parentSegment)
{
}

Episode::Episode(int number, const QString &fName, Segment *parentSegment, const QString &name, qint64 position, Segment::Source source)
  : Segment(position, parentSegment, source),
    m_number(number),
    m_name(name),
    m_fName(fName),
    m_resolution(QSize())
{
}

Episode::~Episode()
{
}

void Episode::read(const QJsonObject &json)
{
  m_number = json["nb"].toInt();
  m_name = json["name"].toString();
  m_fName = json["fName"].toString();
  m_resolution.setWidth(json["width"].toInt());
  m_resolution.setHeight(json["height"].toInt());
  m_fps = json["fps"].toDouble();
  Segment::read(json);
}

void Episode::write(QJsonObject &json) const
{
  json["nb"] = m_number;
  json["name"] = m_name;
  json["fName"] = m_fName;
  json["width"] = m_resolution.width();
  json["height"] = m_resolution.height();
  json["fps"] = m_fps;
  Segment::write(json);
}

QString Episode::display() const
{
  return "Episode " + QString::number(m_number);
}

void Episode::setResolution(const QSize &resolution)
{
  m_resolution = resolution;
}

void Episode::setFps(qreal fps)
{
  m_fps = fps;
}

QSize Episode::getResolution() const
{
  return m_resolution;
}

qreal Episode::getFps() const
{
  return m_fps;
}

QString Episode::getFName() const
{
  return m_fName;
}

int Episode::getNumber() const
{
  return m_number;
}

